/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: David Loffredo (loffredo@steptools.com)
 */


#include <ifc_schema.h>
#include "make_history.h"

//----------------------------------------

// HISTORY UTILITIES -- Create a default owner history and save a
// shortcut to it in the designs object dictionary so that we can
// reuse it when creating IfcRoot instances.
//
// MODIFY THIS AS NEEDED - use person, organization and application
// names appropriate for your own application.
//


IfcOwnerHistory * make_default_owner_history(
    RoseDesign * d
    )
{
    const char * shortcut = "__CONSTANT OWNER HISTORY";

    if (!d) d = ROSE.design();

    // Use shortcut to an owner history if we have already created one
    RoseObject * obj = d-> findObject (shortcut);
    if (obj) return ROSE_CAST (IfcOwnerHistory, obj);


    IfcOwnerHistory * hist = pnewIn(d) IfcOwnerHistory;

    IfcOrganization * org = pnewIn(d) IfcOrganization;
    org-> Name ("Foo Bar, Inc.");

    IfcPerson * person = pnewIn(d) IfcPerson;
    person-> GivenName ("John");
    person-> FamilyName ("Doe");

    IfcPersonAndOrganization * po = pnewIn(d) IfcPersonAndOrganization;
    po-> ThePerson (person);
    po-> TheOrganization (org);

    IfcApplication * app = pnewIn(d) IfcApplication;
    app-> ApplicationDeveloper (org);
    app-> Version ("1.0");
    app-> ApplicationFullName ("sample program");
    app-> ApplicationIdentifier ("program id");

    hist-> OwningUser (po);
    hist-> OwningApplication (app);

    hist-> State (IfcStateEnum_NULL);
    hist-> ChangeAction (IfcChangeActionEnum_NOCHANGE);

    // Optional fields
    hist-> LastModifiedDate (ROSE_NULL_INT);
    hist-> LastModifyingUser (NULL);
    hist-> LastModifyingApplication (NULL);

    // IFC timestamp is seconds since 1970-01-01 (unix epoch)
    hist-> CreationDate ((int) time(0));


    d-> addName (shortcut, hist);  // save shortcut for future calls
    return hist;
}


void assign_ifc_owner_history (IfcRoot * obj)
{
    // simple wrapper to find/create owner history and assign, just to
    // keep the same style as assign_ifc_guid()

    if (obj) obj-> OwnerHistory (
	make_default_owner_history (obj-> design())
	);
}


